#include "thread_calc.h"

Task getNextTask(TaskList& tasks, vector<int>& nodes) {
    Task next_task = tasks.getTask();
    if(next_task.isValid()) return next_task;
    while(!nodes.empty()) {
        int request_node = nodes.back();
        int op_number = OP_REQUEST;
        MPI_Sendrecv_replace(&op_number, 1, MPI_INT, request_node, MSG_REQUEST_TAG, request_node, MSG_RESPONSE_TAG, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
        if(op_number == OP_TASK_NO) {
            nodes.pop_back();
        } else {
            int taskarr[3];
            MPI_Recv(taskarr, 3, MPI_INT, request_node, MSG_TRANSFER_TAG, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            return Task(taskarr[0], taskarr[1], taskarr[2]);
        }
    }
    return INVALID_TASK;
}

void* start_thread_calc(void *arg) {
    TaskList& tasks = *((TaskList*)(arg));
    vector<int> available_nodes;
    for(int i = 0; i < proc_size; i++)
        if(i != proc_rank)
            available_nodes.push_back(i);
    random_shuffle(available_nodes.begin(), available_nodes.end());
    while(true) {
        Task next_task = getNextTask(tasks, available_nodes);
        if(!next_task.isValid()) break;
        cout << "proc #" << proc_rank << " start task #" << next_task.init_thread_id << ":" << next_task.task_number << " (sleep = " << next_task.execution_time << ")" << endl;
        sleep(next_task.execution_time);
    }
    E(proc_rank); Eo("i'm all");
    return NULL;
}
